using System;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;

using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;

using ErrorHandler = Microsoft.VisualStudio.ErrorHandler;
using OleConstants = Microsoft.VisualStudio.OLE.Interop.Constants;
using VSConstants = Microsoft.VisualStudio.VSConstants;

namespace Banquet
{
	[PackageRegistration(UseManagedResourcesOnly = true)]
	[DefaultRegistryRoot(@"Software\Microsoft\VisualStudio\8.0")]
	[InstalledProductRegistration(false, "#100", "#102", "1.0.0.0", IconResourceID = 400)]
	[ProvideLoadKey("Standard", "1.0", "Bazaar", "Canonical", 500)]
	[ProvideMenuResource(1000, 27)]
	[ProvideAutoLoad(GuidList.guidSccProvider)]
	[ProvideService(typeof(SccProviderService), ServiceName = SccProviderService.ServiceName)]
	[ProvideSourceControlProvider(SccProviderService.ProviderName, "#110", GuidList.guidPackage, GuidList.guidSccProvider, GuidList.guidSccProviderService)]
	[ProvideToolWindow(typeof(ExplorerToolWindow), Style = VsDockStyle.MDI, Orientation = ToolWindowOrientation.none, MultiInstances = false, Transient = false)]
	[ProvideToolWindowVisibility(typeof(ExplorerToolWindow), GuidList.guidSccProvider)]
	[ComVisible(true), Guid(GuidList.guidPackage)]
	internal sealed class BanquetPackage : Package, IOleCommandTarget
	{
		private PackageCommandHandler commandHandler;

		public BanquetPackage()
		{
			this.commandHandler = new PackageCommandHandler(this);
			this.commandHandler.RegisterCommands();
		}

		protected override void Initialize()
		{
			base.Initialize();

			IServiceContainer container = (IServiceContainer)GetService(typeof(IServiceContainer));
			container.AddService(typeof(SccProviderService), new SccProviderService(this), true);

			IVsRegisterScciProvider rscp = (IVsRegisterScciProvider)GetService(typeof(IVsRegisterScciProvider));
			rscp.RegisterSourceControlProvider(new Guid(GuidList.guidSccProvider));
		}

		protected override void Dispose(bool disposing)
		{
			try
			{
				SccProviderService service = (SccProviderService)GetService(typeof(SccProviderService));
				if (service != null)
				{
					service.Dispose();
				}
			}
			finally
			{
				base.Dispose(disposing);
			}
		}

		public void AddSecondaryCommandHandler(CommandHandler commandHandler)
		{
			this.commandHandler.AddSecondaryCommandHandler(commandHandler);
		}

		public int OpenToolWindow(Type toolWindowType)
		{
			Debug.Assert(toolWindowType != null);
			ToolWindowPane window = base.FindToolWindow(toolWindowType, 0, true);
			IVsWindowFrame frame = window.Frame as IVsWindowFrame;
			return (frame != null) ? frame.Show() : VSConstants.E_FAIL;
		}

		#region IOleCommandTarget Members

		int IOleCommandTarget.Exec(ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
		{
			IOleCommandTarget target = this.commandHandler;
			return target.Exec(ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
		}

		int IOleCommandTarget.QueryStatus(ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
		{
			IOleCommandTarget target = this.commandHandler;
			return target.QueryStatus(ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
		}

		#endregion
	}
}