using System;
using System.Collections.Generic;
using System.Xml.XPath;

using Microsoft.VisualStudio.VirtualTreeGrid;

namespace Banquet
{
	internal class ExplorerTree : VirtualTree
	{
		private bool dirty;

		public ExplorerTree()
			: base()
		{
		}

		public void Clear()
		{
			ExplorerTreeNode root = new ExplorerTreeNode(ExplorerTreeNodeType.None, null, null, this);
			base.Root = root;
			this.dirty = true;
		}

		public void Load(string path)
		{
			XPathDocument document = new XPathDocument(path);
			XPathNavigator nav = document.CreateNavigator();

			ExplorerTreeNode root = new ExplorerTreeNode(ExplorerTreeNodeType.None, null, null, this);

			Dictionary<string, ExplorerTreeNode> computers = new Dictionary<string, ExplorerTreeNode>();
			ExplorerTreeNode parent;

			if (nav.MoveToFirstChild())
			{
				if (nav.MoveToFirstChild())
				{
					do
					{
						if (nav.NodeType == XPathNodeType.Element && nav.LocalName == "Repository")
						{
							string name = nav.GetAttribute("name", string.Empty);
							Uri location = new Uri(nav.GetAttribute("url", string.Empty));

							if (!computers.TryGetValue(location.Host, out parent))
							{
								string hostname = string.IsNullOrEmpty(location.Host) ? "On this computer" : location.Host;
								parent = new ExplorerTreeNode(ExplorerTreeNodeType.Computer, hostname, null, root);
								root.Add(parent);
							}

							ExplorerTreeNode node = new ExplorerTreeNode(ExplorerTreeNodeType.Repository, name, location, parent);
							parent.Add(node);
							Load(nav, node);
							break;
						}
					}
					while (nav.MoveToNext());
					nav.MoveToParent();
				}
			}

			base.Root = root;
			this.dirty = false;
		}

		private void Load(XPathNavigator nav, ExplorerTreeNode parent)
		{
			if (nav.MoveToFirstChild())
			{
				do
				{
					if (nav.NodeType == XPathNodeType.Element)
					{
						switch (nav.LocalName)
						{
						case "Branch":
							{
								string name = nav.GetAttribute("name", string.Empty);
								Uri location = new Uri(nav.GetAttribute("url", string.Empty));
								ExplorerTreeNode node = new ExplorerTreeNode(ExplorerTreeNodeType.Branch, name, location, parent);
								parent.Add(node);
								break;
							}
						case "Directory":
							{
								string name = nav.GetAttribute("name", string.Empty);
								ExplorerTreeNode dir = new ExplorerTreeNode(ExplorerTreeNodeType.Directory, name, null, parent);
								parent.Add(dir);
								Load(nav, dir);
								break;
							}
						}
					}
				}
				while (nav.MoveToNext());
				nav.MoveToParent();
			}
		}

		public bool IsDirty
		{
			get
			{
				return this.dirty;
			}
		}

		public new ExplorerTreeNode Root
		{
			get
			{
				return (ExplorerTreeNode)base.Root;
			}
		}
	}
}