using System;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Banquet
{
	internal class ExplorerTreeComboBox : TreeComboBox
	{
		private bool hideBranches;

		public ExplorerTreeComboBox()
		{
			ImageList imageList = new ImageList();
			imageList.ColorDepth = ColorDepth.Depth24Bit;
			imageList.ImageSize = new Size(16, 16);
			imageList.TransparentColor = Color.Magenta;
			imageList.Images.AddStrip(GuiResources.listviewicons);

			base.ImageList = imageList;
			base.DropDownStyle = ComboBoxStyle.DropDownList;
		}

		public bool HideBranches
		{
			get
			{
				return this.hideBranches;
			}
			set
			{
				this.hideBranches = value;
			}
		}

		public void Fill(ExplorerTree tree)
		{
			base.BeginUpdate();
			try
			{
				base.Items.Clear();

				foreach (ExplorerTreeNode child in tree.Root.Children)
				{
					Fill(child, 0);
				}
			}
			finally
			{
				base.EndUpdate();
			}
		}

		private void Fill(ExplorerTreeNode node, int indentLevel)
		{
			if (!this.hideBranches || node.Type <= ExplorerTreeNodeType.Repository || node.Type == ExplorerTreeNodeType.Directory)
			{
				TreeComboBoxItem item = new TreeComboBoxItem(node.Name, indentLevel, (int)node.GetImageIndex(true));
				item.Tag = node;
				base.Items.Add(item);

				indentLevel++;
				foreach (ExplorerTreeNode child in node.Children)
				{
					Fill(child, indentLevel);
				}
			}
		}

		protected override string GetComboEditText(TreeComboBoxItem item)
		{
			ExplorerTreeNode node = item.Tag as ExplorerTreeNode;

			if (node != null)
			{
				StringBuilder s = new StringBuilder(node.Name);

				if (node.Parent != null)
				{
					node = node.Parent;
					while (node.Parent != null)
					{
						s.Insert(0, '/').Insert(0, node.Name);
						node = node.Parent;
					}
				}

				return s.ToString();
			}

			return base.GetComboEditText(item);
		}
	}
}