using System;
using System.ComponentModel;
using System.Windows.Forms;

using Microsoft.VisualStudio.VirtualTreeGrid;

namespace Banquet
{
	internal class TreeGrid : VirtualTreeControl
	{
		#region Fields

		private BorderStyle borderStyle;

		#endregion

		#region Constructors

		public TreeGrid()
		{
			this.borderStyle = BorderStyle.Fixed3D;
		}

		#endregion

		#region Properties

		[DefaultValue(BorderStyle.Fixed3D)]
		public virtual BorderStyle BorderStyle
		{
			get
			{
				return this.borderStyle;
			}
			set
			{
				if (this.borderStyle != value)
				{
					this.borderStyle = value;
					base.Invalidate(true);
				}
			}
		}

		protected override CreateParams CreateParams
		{
			get
			{
				CreateParams createParams = base.CreateParams;

				switch (this.borderStyle)
				{
				case BorderStyle.None:
					createParams.Style &= ~NativeMethods.WS_BORDER;
					createParams.ExStyle &= ~NativeMethods.WS_EX_CLIENTEDGE;
					break;

				case BorderStyle.FixedSingle:
					createParams.Style |= NativeMethods.WS_BORDER;
					createParams.ExStyle &= ~NativeMethods.WS_EX_CLIENTEDGE;
					break;

				case BorderStyle.Fixed3D:
					createParams.Style &= ~NativeMethods.WS_BORDER;
					createParams.ExStyle |= NativeMethods.WS_EX_CLIENTEDGE;
					break;
				}

				return createParams;
			}
		}

		#endregion
	}
}