using System;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Windows.Forms;

using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;

using ErrorHandler = Microsoft.VisualStudio.ErrorHandler;

namespace Banquet
{
	internal abstract class ToolWindowPaneBase : ToolWindowPane
	{
		private ToolWindowControlBase toolWindowBase;
		private string caption;

		public ToolWindowPaneBase(ToolWindowControlBase toolWindow)
			: base(null)
		{
			Debug.Assert(toolWindow != null);
			this.toolWindowBase = toolWindow;
			this.toolWindowBase.TextChanged += TextChanged;
		}

		protected override void Dispose(bool disposing)
		{
			try
			{
				if (disposing)
				{
					IVsWindowFrame windowFrame = (IVsWindowFrame)this.Frame;
					if (windowFrame != null)
					{
						windowFrame.CloseFrame((uint)__FRAMECLOSE.FRAMECLOSE_SaveIfDirty);
					}
				}
			}
			finally
			{
				base.Dispose(disposing);
			}
		}

		public void Hide()
		{
			IVsWindowFrame windowFrame = (IVsWindowFrame)this.Frame;
			if (windowFrame != null)
			{
				windowFrame.Hide();
			}
		}

		protected virtual void InitializeToolWindowUI()
		{
			this.toolWindowBase.InitializeToolWindowUI();
		}

		public override void OnToolWindowCreated()
		{
			base.OnToolWindowCreated();
			InitializeToolWindowUI();
		}

		public void Show()
		{
			IVsWindowFrame windowFrame = (IVsWindowFrame)this.Frame;
			if (windowFrame != null)
			{
				windowFrame.Show();
			}
		}

		public void ShowNoActivate()
		{
			IVsWindowFrame windowFrame = (IVsWindowFrame)this.Frame;
			if (windowFrame != null)
			{
				windowFrame.ShowNoActivate();
			}
		}

		private void TextChanged(object sender, EventArgs e)
		{
			if (this.caption == null)
			{
				this.caption = base.Caption ?? string.Empty;
			}

			if (string.IsNullOrEmpty(this.toolWindowBase.Text))
			{
				base.Caption = this.caption;
			}
			else
			{
				base.Caption = string.Concat(this.caption, " - ", ((ToolWindowControlBase)sender).Text);
			}
		}

		public bool IsOnScreen
		{
			get
			{
				int pfOnScreen = 0;
				IVsWindowFrame windowFrame = (IVsWindowFrame)this.Frame;
				if (windowFrame != null)
				{
					windowFrame.IsOnScreen(out pfOnScreen);
				}
				return (pfOnScreen != 0);
			}
		}

		public override IWin32Window Window
		{
			get
			{
				return this.toolWindowBase;
			}
		}
	}
}