using System;
using System.IO;
using System.Runtime.Serialization;

using Python;

namespace Canonical.Bazaar
{
	public class BazaarException : Exception, ISerializable
	{
		public BazaarException()
		{
		}

		public BazaarException(string message)
			: base(message)
		{
		}

		public BazaarException(string message, Exception innerException)
			: base(message, innerException)
		{
		}

		protected BazaarException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}

		internal static void Handle(PythonException e)
		{
			if (e.Type.Module == "bzrlib.errors")
			{
				HandleError(e);
			}
		}

		private static void HandleError(PythonException e) // TODO: should BazaarExceptions have e as InnerException?
		{
			switch (e.Type.Name)
			{
			case "BzrMoveFailedError":
				using (PythonString from_path = e.Value.GetAttribute<PythonString>("from_path"))
				using (PythonString to_path = e.Value.GetAttribute<PythonString>("to_path"))
				using (PythonString extra = e.Value.GetAttribute<PythonString>("extra"))
				{
					throw new MoveFailedException(from_path.ToString(), to_path.ToString());
				}

			case "BzrRenameFailedError":
				using (PythonString from_path = e.Value.GetAttribute<PythonString>("from_path"))
				using (PythonString to_path = e.Value.GetAttribute<PythonString>("to_path"))
				using (PythonString extra = e.Value.GetAttribute<PythonString>("extra"))
				{
					throw new RenameFailedException(from_path.ToString(), to_path.ToString());
				}

			case "DivergedBranches":
				throw new DivergedBranchesException();

			case "NoRepositoryPresent":
				using (PythonString path = e.Value.GetAttribute<PythonString>("path"))
				{
					throw new NoRepositoryPresentException(Path.GetFullPath(path.ToString()));
				}

			case "NoSuchTag":
				using (PythonString tag_name = e.Value.GetAttribute<PythonString>("tag_name"))
				{
					throw new NoSuchTagException(tag_name.ToString());
				}

			case "NotBranchError":
				using (PythonString path = e.Value.GetAttribute<PythonString>("path"))
				{
					throw new NotBranchException(Path.GetFullPath(path.ToString()));
				}

			case "NoWorkingTree":
				using (PythonString @base = e.Value.GetAttribute<PythonString>("base"))
				{
					throw new NoWorkingTreeException(new Uri(@base.ToString()));
				}

			case "PathNotChild":
				using (PythonString path = e.Value.GetAttribute<PythonString>("path"))
				using (PythonString @base = e.Value.GetAttribute<PythonString>("base"))
				{
					throw new PathNotChildException(Path.GetFullPath(path.ToString()), Path.GetFullPath(@base.ToString()));
				}

			case "TagAlreadyExists":
				using (PythonString tag_name = e.Value.GetAttribute<PythonString>("tag_name"))
				{
					throw new TagAlreadyExistsException(tag_name.ToString());
				}
			}
		}
	}
}