using System;
using System.Collections.Generic;

namespace Python
{
	public class PythonList : PythonSequence
	{
		public PythonList()
		{
		}

		public static PythonList Create()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyList_New(0));
		}

		public static PythonList Create(PythonObject item0)
		{
			if ((object)item0 == null)
			{
				throw new ArgumentNullException("item0");
			}

			PythonList list = PythonException.ThrowIfInvalid(NativeMethods.PyList_New(1));
			PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, 0, item0));
			return list;
		}

		public static PythonList Create(PythonObject item0, PythonObject item1)
		{
			if ((object)item0 == null)
			{
				throw new ArgumentNullException("item0");
			}
			if ((object)item1 == null)
			{
				throw new ArgumentNullException("item1");
			}

			PythonList list = PythonException.ThrowIfInvalid(NativeMethods.PyList_New(2));
			PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, 0, item0));
			PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, 1, item1));
			return list;
		}

		public static PythonList Create(PythonObject item0, PythonObject item1, PythonObject item2)
		{
			if ((object)item0 == null)
			{
				throw new ArgumentNullException("item0");
			}
			if ((object)item1 == null)
			{
				throw new ArgumentNullException("item1");
			}
			if ((object)item2 == null)
			{
				throw new ArgumentNullException("item2");
			}

			PythonList list = PythonException.ThrowIfInvalid(NativeMethods.PyList_New(3));
			PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, 0, item0));
			PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, 1, item1));
			PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, 2, item2));
			return list;
		}

		public static PythonList Create(params PythonObject[] items)
		{
			if (items == null)
			{
				throw new ArgumentNullException("items");
			}

			PythonList list = PythonException.ThrowIfInvalid(NativeMethods.PyList_New(items.Length));

			for (int i = 0; i < items.Length; i++)
			{
				if ((object)items[i] == null)
				{
					throw new ArgumentException();
				}

				PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, i, items[i]));
			}

			return list;
		}

		public static PythonList Create(IList<PythonObject> items)
		{
			if (items == null)
			{
				throw new ArgumentNullException("items");
			}

			PythonList list = PythonException.ThrowIfInvalid(NativeMethods.PyList_New(items.Count));

			int index = 0;

			foreach (PythonObject item in items)
			{
				if ((object)item == null)
				{
					throw new ArgumentException();
				}

				PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, index, item));
				index++;
			}

			return list;
		}

		public static PythonList Create(IList<string> items)
		{
			if (items == null)
			{
				throw new ArgumentNullException("items");
			}

			PythonList list = PythonException.ThrowIfInvalid(NativeMethods.PyList_New(items.Count));

			int index = 0;

			foreach (string item in items)
			{
				if (item == null)
				{
					throw new ArgumentException();
				}

				PythonException.ThrowIfNegative(NativeMethods.PyList_SetItem(list, index, item));
				index++;
			}

			return list;
		}

		public void Append(PythonObject item)
		{
			if ((object)item == null)
			{
				throw new ArgumentNullException("item");
			}

			PythonException.ThrowIfNegative(NativeMethods.PyList_Append(this, item));
		}

		public void Insert(int index, PythonObject item)
		{
			if ((object)item == null)
			{
				throw new ArgumentNullException("item");
			}

			PythonException.ThrowIfNegative(NativeMethods.PyList_Insert(this, index, item));
		}

		public void Reverse()
		{
			PythonException.ThrowIfNegative(NativeMethods.PyList_Reverse(this));
		}

		public void Sort()
		{
			PythonException.ThrowIfNegative(NativeMethods.PyList_Sort(this));
		}
	}
}