using System;
using System.Runtime.InteropServices;

namespace Python
{
	public class PythonString : PythonSequence
	{
		public PythonString()
		{
		}

		public PythonString Join(PythonIterable<PythonString> list)
		{
			return InvokeMethod<PythonString>("join", list);
		}

		public PythonString Join(PythonList list)
		{
			return InvokeMethod<PythonString>("join", list);
		}

		public static PythonString CreateString(string value)
		{
			if (value == null)
			{
				throw new ArgumentNullException("value");
			}

			return PythonException.ThrowIfInvalid(NativeMethods.PyString_FromStringAndSize(value, value.Length));
		}

		public static PythonString CreateUnicode(string value)
		{
			if (value == null)
			{
				throw new ArgumentNullException("value");
			}

			return PythonException.ThrowIfInvalid(NativeMethods.PyUnicode_FromUnicode(value, value.Length));
		}

		public unsafe override string ToString()
		{
			PythonType type = this.Type;
			string value;

			if (type == StringType)
			{
				value = Marshal.PtrToStringAnsi(NativeMethods.PyString_AsString(this), NativeMethods.PyString_Size(this));
				PythonException.ThrowIfErrorOccurred();
			}
			else if (type == UnicodeType)
			{
				value = new string(NativeMethods.PyUnicode_AsUnicode(this), 0, NativeMethods.PyUnicode_GetSize(this));
				PythonException.ThrowIfErrorOccurred();
			}
			else if (type == NoneType)
			{
				value = null;
			}
			else
			{
				throw new InvalidOperationException();
			}

			return value;
		}
	}
}