using System;

namespace Python
{
	public class PythonTuple : PythonSequence
	{
		public PythonTuple()
		{
		}

		public static PythonTuple Create()
		{
			return PythonException.ThrowIfInvalid(NativeMethods.PyTuple_New(0));
		}

		public static PythonTuple Create(PythonObject item0)
		{
			if ((object)item0 == null)
			{
				throw new ArgumentNullException("item0");
			}

			PythonTuple tuple = PythonException.ThrowIfInvalid(NativeMethods.PyTuple_New(1));
			PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, 0, item0));
			return tuple;
		}

		public static PythonTuple Create(PythonObject item0, PythonObject item1)
		{
			if ((object)item0 == null)
			{
				throw new ArgumentNullException("item0");
			}
			if ((object)item1 == null)
			{
				throw new ArgumentNullException("item1");
			}

			PythonTuple tuple = PythonException.ThrowIfInvalid(NativeMethods.PyTuple_New(2));
			PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, 0, item0));
			PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, 1, item1));
			return tuple;
		}

		public static PythonTuple Create(PythonObject item0, PythonObject item1, PythonObject item2)
		{
			if ((object)item0 == null)
			{
				throw new ArgumentNullException("item0");
			}
			if ((object)item1 == null)
			{
				throw new ArgumentNullException("item1");
			}
			if ((object)item2 == null)
			{
				throw new ArgumentNullException("item2");
			}

			PythonTuple tuple = PythonException.ThrowIfInvalid(NativeMethods.PyTuple_New(3));
			PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, 0, item0));
			PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, 1, item1));
			PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, 2, item2));
			return tuple;
		}

		public static PythonTuple Create(params PythonObject[] items)
		{
			if (items == null)
			{
				throw new ArgumentNullException("items");
			}

			PythonTuple tuple = PythonException.ThrowIfInvalid(NativeMethods.PyTuple_New(items.Length));

			for (int i = 0; i < items.Length; i++)
			{
				if ((object)items[i] == null)
				{
					throw new ArgumentException();
				}

				PythonException.ThrowIfNegative(NativeMethods.PyTuple_SetItem(tuple, i, items[i]));
			}

			return tuple;
		}
	}
}