using System;
using System.Runtime.InteropServices;

namespace Python
{
	public class PythonType : PythonObject
	{
		private string name;
		private string module;

		public PythonType()
		{
		}

		public unsafe string Name
		{
			get
			{
				if (base.IsInvalid)
				{
					throw new InvalidOperationException();
				}

				if (this.name == null)
				{
					NativeMethods.PyTypeObject* op = (NativeMethods.PyTypeObject*)base.handle;
					this.name = Marshal.PtrToStringAnsi(op->tp_name);
				}

				return this.name;
			}
		}

		public string Module
		{
			get
			{
				if (base.IsInvalid)
				{
					throw new InvalidOperationException();
				}

				if (this.module == null)
				{
					using (PythonString s = PythonException.ThrowIfInvalid(NativeMethods.PyObject_GetAttrString<PythonString>(this, "__module__")))
					{
						this.module = s.ToString();
					}
				}

				return this.module;
			}
		}

		public bool IsSubtypeOf(PythonType type)
		{
			if ((object)type == null)
			{
				throw new ArgumentNullException("type");
			}
			if (base.IsInvalid || type.IsInvalid)
			{
				throw new InvalidOperationException();
			}
			if (base.handle == type.handle)
			{
				return true;
			}

			return PythonException.ThrowIfNegative(NativeMethods.PyType_IsSubtype(this, type)) > 0;
		}
	}
}